/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file ffstream.cc implements ffstream.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif


// std c
#include <stdio.h>

// std c++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <string>
using std::string;

// this header
#include "ffistream.hh"

#include "ffmpeg.hh"

// PIAVE
#include "libpiave/frame.hh"

using namespace PIAVE;

/*
 *  \class InFFMpegStream 
 *
 */
InFFMpegStream::InFFMpegStream( const string& fileName )
  : InAVFStream( fileName ),
    _avCodec( 0 ),
    _avCodecContext( 0 )
{
    FFMpeg::init();
    initDecoder();
    return;
}

InFFMpegStream::~InFFMpegStream()
{

    WARN( "FIXME" );

//     fclose(f);

//     avcodec_close(c);
//     free(c);
//     printf("\n");
}


void
InFFMpegStream::initDecoder()
{
    INFO( "Init InFFMpegStream " );
    /*
     *    I cut&past this from libavcodec apiexample
     *    and I have no idea what I am doing.....
     */

    /* find the mpeg1 video decoder */
    _avCodec = avcodec_find_decoder( CODEC_ID_MPEG1VIDEO );
    if ( !_avCodec ) {
        ABORT( "codec not found" );
    }

    /* what is this ?? */
//    _avCodecContext = avcodec_alloc_context();

    /* for some codecs, such as msmpeg4 and mpeg4, width and height
       MUST be initialized there because these info are not available
       in the bitstream */

    /* open it */
    if ( avcodec_open( _avCodecContext, _avCodec ) < 0 ) {
        ABORT( "could not open codec" );
    }
    
    /* the codec gives us the frame size, in samples */
    _file = fopen( getFileName().c_str(), "r");
    if (!_file) {
        ABORT( "could not open file" );
    }

    /*
     *   read info from file
     */


    AVOutputFormat * _avFormat = guess_format( 0 , getFileName().c_str(), 0 );
    INFO( "Guessed:" );
    INFO( PRINTV( _avFormat->name ));
    INFO( PRINTV( _avFormat->long_name ));
    INFO( PRINTV( _avFormat->mime_type ));
    INFO( PRINTV( _avFormat->extensions ));
    INFO( PRINTV( _avFormat->priv_data_size ));

    setLength( 10 );

    return;
}

Frame
InFFMpegStream::getFrame( Time t )
{
    Frame f;
    static uint8_t inbuf[ FFMPEG_INBUF_SIZE ];

    int size = fread( inbuf, 1, FFMPEG_INBUF_SIZE, _file );

    if (size == 0) {
        WARN( "Arg.... nothing to read:" );
        return f;
    }

    /* NOTE1: some codecs are stream based (mpegvideo, mpegaudio)
       and this is the only method to use them because you cannot
       know the compressed data size before analysing it. 
       
       BUT some other codecs (msmpeg4, mpeg4) are inherently frame
       based, so you must call them with all the data for one
       frame exactly. You must also initialize 'width' and
       'height' before initializing them. */
    
    /* NOTE2: some codecs allow the raw parameters (frame size,
       sample rate) to be changed at any frame. We handle this, so
       you should also take care of it */
    
    /* here, we use a stream based decoder (mpeg1video), so we
       feed decoder and see if it could decode a frame */

    int got_picture, len;

    uint8_t *inbuf_ptr = inbuf;


    len = avcodec_decode_video( _avCodecContext, 
                                &_picture, &got_picture, 
                                inbuf_ptr, size);
    if (len < 0) {
        ABORT( "Error while decoding frame " );
    } else {
        INFO( "avdecode return "<<PRINTV(len) );
    }

    if (got_picture) {
        INFO( " ;-) YEP! GOT A PICTURE" );
    } else {
        WARN( " ;-( Couldn't get picture " );
    }

    /* the picture is allocated by the decoder. no need to
       free it */

    // FIXME - save position... well, do it right
    //size -= len;
    //inbuf_ptr += len;

    /* some codecs, such as MPEG, transmit the I and P frame with a
       latency of one frame. You must do the following to have a
       chance to get the last frame of the video */
//     len = avcodec_decode_video(c, &picture, &got_picture, 
//                                NULL, 0);
//     if (got_picture) {
//         printf("saving frame %3d\r", frame);
//         fflush(stdout);
        
//         /* the picture is allocated by the decoder. no need to
//            free it */
//         snprintf(buf, sizeof(buf), outfilename, frame);
//         pgm_save(picture.data[0], picture.linesize[0], 
//                  c->width, c->height, buf);
//         frame++;
//     }
    return f;
}



